type op = Add|Sub|Mult|Div|Equal|Neq|Less|Leq|Greater| Geq| Dadd| Dsub|Dmult| Ddiv| And| Or
type expr =
	Integer of int
| String  of string
| Array  of expr list
| Id 			of  string
| Binop   of expr * op * expr
| Assign  of string * expr
| Call    of string * expr list
| Index   of string * expr
| Noexpr
(*type constant_type =
	Integer of int
	|String of string
	| Array of expr list
*)


type stmt =
    Block of stmt list
  | Expr of expr
  | Return of expr
  | If of expr * stmt * stmt
  | While of expr * stmt

type variable_decl = {
	 vname: string;
	 vtype: Types.t;
	 (*value: expr;*)
	 vsize: int;
}

type func_decl = {
  fname : string;
  formals : variable_decl list;
  locals : variable_decl list;
  body : stmt list;
  returntype : Types.t;
}

type program = variable_decl list * func_decl list

(*test if the parser and ast is correct*)
let string_of_op = function
    Add -> "+"
  | Sub -> "-"
  | Mult -> "*"
  | Div -> "/"
  | Equal -> "=="
  | Neq -> "!="
  | Less -> "<"
  | Leq -> "<=" 
  | Greater -> ">" 
  | Geq -> ">="
  | And -> "&&"
  | Or -> "||"
  | Dadd ->".+"
  | Dsub -> ".-"
  | Dmult ->".*"
  | Ddiv -> "./"

let rec string_of_expr = function 
	  Integer(i) -> string_of_int i
	| String(s) -> s
	| Array(a) ->  "("^ String.concat "," (List.map string_of_expr a)^")" 
	| Id (i) -> i 
	| Binop(e1,o,e2) -> string_of_expr e1 ^" "^ string_of_op o^" "^ string_of_expr e2   
	| Assign(l, e) -> l ^ "=" ^ string_of_expr e
	| Call( f, e) -> f^ "(" ^String.concat ", " (List.map string_of_expr e) ^ ")" 
	| Index(i, e) -> i ^ "[" ^ string_of_expr e  ^ "]"
	| Noexpr ->""
let rec string_of_obj_type t =match t with 
  Types.Int-> "int"
| Types.Str ->"string"
| Types.Arr -> "array"

let string_of_vdecl id = 
  string_of_obj_type id.vtype ^ " " ^ id.vname ^ " = " ^ string_of_int id.vsize ^ ";\n"

let rec string_of_stmt = function
    Block(stmts) ->
      "{\n" ^ String.concat "" (List.map string_of_stmt stmts) ^ "}\n"
  | Expr(e) -> string_of_expr e ^ ";\n";
  | Return(e) -> "return " ^ string_of_expr e ^ ";\n";
  | If(e, s1, s2) ->  "if (" ^ string_of_expr e ^ ")\n" ^
      string_of_stmt s1 ^ "else\n" ^ string_of_stmt s2
  | While(e, s) -> "while (" ^ string_of_expr e ^ ") " ^ string_of_stmt s

let string_of_fdecl fdecl =
  string_of_obj_type fdecl.returntype ^ " " ^
  fdecl.fname ^ "(" ^ String.concat ", " (List.map string_of_vdecl
  fdecl.formals) ^ ")\n{\n" ^
  String.concat "" (List.map string_of_vdecl fdecl.locals) ^
  String.concat "" (List.map string_of_stmt fdecl.body) ^
  "}\n"
	
let string_of_program (vars, funcs) =
  String.concat "" (List.map string_of_vdecl vars) ^ "\n" ^
  String.concat "\n" (List.map string_of_fdecl funcs)