open Ast
open Bytecode

let execute_prog prog =
let stack = Array.make 8192 0
and globals = Array.make 8192 0 in

let rec exec fp sp pc = match prog.text.(pc) with

Litin i -> stack.(sp) <- i ; exec fp (sp+1) (pc+1)

|Litsh str -> (let rec push_str_elements str' sl =
				if sl >= 0 then ( stack.(sp+(String.length str)-1-sl) <- int_of_char str'.[sl]; 
						push_str_elements str' (sl-1);)
		in push_str_elements str ((String.length str)-1));
		exec fp (sp+(String.length str)) (pc+1)		


| Drp -> exec fp (sp-1) (pc+1)
| Bin op -> if (op=Add ||op=Sub||op=Mult||op=Div||op=Equal||op=Neq||op=Less||op=Leq||op=Greater||op=Geq||op=And||op=Or) then 
	(let op1 = stack.(sp-2) and op2 = stack.(sp-1) in
  stack.(sp-2) <- (let boolean i = if i then 1 else 0 in
match op with
Add -> op1 + op2
| Sub -> op1 - op2
| Mult -> op1 * op2
| Div -> op1 / op2
| Equal -> boolean (op1 = op2)
| Neq -> boolean (op1 != op2)
| Less -> boolean (op1 < op2)
| Leq -> boolean (op1 <= op2)
| Greater -> boolean (op1 > op2)
| Geq -> boolean (op1 >= op2)
| And -> if (op1<>0 &&op2<>0) then 1 else 0
| Or  -> if (op1=0 && op2 =0)  then 0 else 1);

exec fp (sp-1) (pc+1)
)
else
((match op with
Dmult -> (let rec d_mult offset =
			if offset>=0 then (stack.(sp-3-offset) <- stack.(sp-3-offset) * stack.(sp-1);
					    d_mult (offset-1))
			in d_mult (stack.(sp-2)-1))

|Dadd-> (let rec d_add offset =
			if offset>=0 then (stack.(sp-3-offset) <- 					stack.(sp-3-offset) + stack.(sp-1);
					    d_add (offset-1))
			in d_add (stack.(sp-2)-1))

|Dsub-> (let rec d_sub offset =
			if offset>=0 then (stack.(sp-3-offset) <- 					stack.(sp-3-offset) - stack.(sp-1);
					    d_sub (offset-1))
			in d_sub (stack.(sp-2)-1))
|Ddiv-> (let rec d_div offset =
			if offset>=0 then (stack.(sp-3-offset) <- 					stack.(sp-3-offset) / stack.(sp-1);
					    d_div (offset-1))
			in d_div (stack.(sp-2)-1))


);exec fp (sp-2) (pc+1))

					    


|Lod i-> stack.(sp) <- globals.(i) ; exec fp (sp+1) (pc+1)

|Loda i ->  (  let length=stack.(sp-1) in
		let rec load_arr_globals i' offset=
			if offset >= 0 then (stack.(sp-2+length-offset) <- globals.(i'-offset);
					     load_arr_globals i' (offset-1))
		in (load_arr_globals i (stack.(sp-1)-1);exec fp (sp+length-1) (pc+1)))


|Str i-> globals.(i) <- stack.(sp-1) ; exec fp sp (pc+1)

|Stra i -> 	(let rec push_arr_globals i' offset =
			if offset >= 0 then (globals.(i'-offset) <- stack.(sp-2-offset);
					     push_arr_globals i' (offset-1))
 		in push_arr_globals i (stack.(sp-1)-1));

		exec fp (sp-1) (pc+1);




|Lfp i-> stack.(sp) <- stack.(fp+i) ; exec fp (sp+1) (pc+1)

|Lfpa i ->  (  let length=stack.(sp-1) in
		let rec load_arr_elements i' offset=
			if offset >= 0 then (stack.(sp-2+length-offset) <- stack.(fp+i'-offset);
					     load_arr_elements i' (offset-1))
		in (load_arr_elements i (stack.(sp-1)-1);exec fp (sp+length-1) (pc+1)))



|GetC -> stack.(sp) <- stack.(sp-1-stack.(sp-1)-1); exec fp (sp+1) (pc+1)


|Sfp i-> stack.(fp+i) <- stack.(sp-1) ; exec fp sp (pc+1)

|Sfpa i-> (let rec push_arr_elements i' offset =
			if offset >= 0 then (stack.(fp+i'-offset) <- stack.(sp-2-offset);
					     push_arr_elements i' (offset-1))
	 	in push_arr_elements i (stack.(sp-1)-1));
	exec fp (sp-1) (pc+1);		


|Jsr(-1)-> print_endline (string_of_int stack.(sp-1)) ; exec fp sp (pc+1)

|Jsr(-2) -> (let rec print_array offset =
			if offset >= 0 then (print_endline (string_of_int (stack.(sp-2-(stack.(sp-1)-1-offset))));
					     print_array (offset-1))
	 	in print_array (stack.(sp-1)-1) );
	exec fp sp (pc+1);
|Jsr(-3) -> (let rec print_string offset =
			if offset >= 0 then (print_char (char_of_int (stack.(sp-2-(stack.(sp-1)-1-offset))));
					     print_string (offset-1))
	 	in print_string (stack.(sp-1)-1) );
	exec fp sp (pc+1);


| Jsr i -> stack.(sp) <- pc + 1 ; exec fp (sp+1) i
| Ent i -> stack.(sp) <- fp ; exec sp (sp+i+1) (pc+1)
| Rts i -> let new_fp = stack.(fp) and new_pc = stack.(fp-1) in stack.(fp-i-1) <- stack.(sp-1) ;exec new_fp (fp-i) new_pc
| Beq i -> exec fp (sp-1) (pc + if stack.(sp-1) = 0 then i else 1)
| Bne i -> exec fp (sp-1) (pc + if stack.(sp-1) != 0 then i else 1)
| Bra i -> exec fp sp (pc+i)
| Hlt -> ()
in exec 0 0 0





















