%{ open Ast%}
%{ open Types%}

%token ASSIGN LPAREN RPAREN LBRACKET RBRACKET LBRACE RBRACE SEMI COMMA PERCENT  DEF EOF 
%token IF WHILE ELSE INT STR INTARRAY RETURN 
%token PLUS MINUS TIMES DIVIDE DADD DSUB DMULT DDIV AND OR EQ NEQ GT LT GEQ LEQ
%token INTARRAY CREATEARRAY
%token <int> INTEGER
%token <string> STRING ID
%nonassoc NOELSE
%nonassoc ELSE
%right ASSIGN
%left EQ NEQ
%left OR 
%left AND
%left LT GT LEQ GEQ
%left PLUS MINUS DADD DSUB 
%left TIMES DIVIDE DMULT DDIV
%start program
%type <Ast.program> program
%%
program:
/* nothing */ { [], [] }
| program vdecl { ($2 :: fst $1), snd $1 }
| program fdecl { fst $1, ($2 :: snd $1) }
fdecl:
DEF types ID LPAREN formals_opt RPAREN LBRACE vdecl_list stmt_list RBRACE
{ { fname = $3;
formals = $5;
locals = List.rev $8;
body = List.rev $9;
returntype = $2; } }
formals_opt:
/* nothing */ { [] }
| formal_list { List.rev $1 }

formal_decl:
	types ID  {($2, $1)} 
formal_list:
  formal_decl { [{vname=fst $1; vtype=snd $1;vsize=1;}]}
| formal_list COMMA formal_decl{ {vname= fst $3; vtype=snd $3;vsize=1;}::$1}

types:
  INT     {Types.Int}
| STR     {Types.Str}
| INTARRAY   {Types.Arr}


vdecl_list:
/* nothing */ { [] }
| vdecl_list vdecl { $2 :: $1 }
vdecl:
  INT ID SEMI { {vname = $2; vtype=Types.Int;  vsize=1;} }
| STR LBRACKET INTEGER RBRACKET ID SEMI { {vname = $5; vtype=Types.Str; vsize=$3;} }
|INTARRAY LBRACKET INTEGER RBRACKET ID SEMI {{vname = $5; vtype=Types.Arr;vsize =$3;}}
/*| INTARRAY ID ASSIGN listvalue  SEMI { {vname = $2; vtype=Types.Arr; vvalue=Array(fst $4);vsize = snd $4;} }*/
stmt_list:
/* nothing */ { [] }
| stmt_list stmt { $2 :: $1 }

stmt:
expr SEMI { Expr($1) }
| RETURN expr SEMI { Return($2) }
| LBRACE stmt_list RBRACE { Block(List.rev $2) }
| IF LPAREN expr RPAREN stmt %prec NOELSE { If($3, $5, Block([])) }
| IF LPAREN expr RPAREN stmt ELSE stmt { If($3, $5, $7) }
| WHILE LPAREN expr RPAREN stmt { While($3, $5) }
expr:
  INTEGER { Integer($1) }
| STRING {  String ($1)}
| ID { Id($1) }
| expr PLUS expr { Binop($1, Add, $3) }
| expr MINUS expr { Binop($1, Sub, $3) }
| expr TIMES expr { Binop($1, Mult, $3) }
| expr DIVIDE expr { Binop($1, Div, $3) }
| expr EQ expr { Binop($1, Equal, $3) }
| expr NEQ expr { Binop($1, Neq, $3) }

| expr LT expr { Binop($1, Less, $3) }
| expr LEQ expr { Binop($1, Leq, $3) }
| expr GT expr { Binop($1, Greater, $3) }
| expr GEQ expr { Binop($1, Geq, $3) }
| ID ASSIGN expr { Assign($1, $3) }
| ID LPAREN actuals_opt RPAREN { Call($1, $3) }
| LPAREN expr RPAREN { $2 }
| listvalue  {Array(List.rev(fst $1))}
/*| ID LBRACKET INTEGER RBRACKET {Index($1,Integer($3))} */ /*match index*/
| ID LBRACKET expr RBRACKET {Index($1,$3)} 

/*our version*/
| expr DADD expr  {Binop($1, Dadd, $3)}
| expr DSUB expr  {Binop($1, Dsub, $3)}
| expr DMULT expr {Binop($1, Dmult, $3)} 
| expr DDIV  expr {Binop($1, Ddiv, $3)}
| expr AND expr {Binop($1, And, $3)}
| expr OR  expr {Binop($1, Or, $3)}

listvalue:
PERCENT LPAREN listintelement RPAREN PERCENT{(fst $3,snd $3)}   /*int array*/



listintelement:
| INTEGER  {([Integer($1)],1)}
| listintelement COMMA INTEGER {(Integer($3)::(fst $1),snd $1 + 1)}



actuals_opt:
/* nothing */ { [] }
| actuals_list { List.rev $1 }
actuals_list:
expr { [$1] }
| actuals_list COMMA expr { $3 :: $1 }