open Types
open Ast 
type expr_detail =
    Integer of int
	| String of string
  | Id of variable_decl
  | Binop of expr * Ast.op * expr
  | Assign of variable_decl * expr
  | Call of function_decl * expr list
  | Array of expr list
  | Index of variable_decl * expr
  | Noexpr
and expr = expr_detail * Types.t

and stmt =
    Block of stmt list
  | Expr of expr
  | Return of expr
  | If of expr * stmt * stmt
  | While of expr * stmt

and  variable_decl = {
	v_name : string;
	v_type : Types.t;
	(*v_value: Ast.expr;*)
	v_size : int;
}	

and function_decl = {
  ffname : string;
  fformals : variable_decl list;
  flocals :  variable_decl list;
  fbody : stmt list;
  freturntype : Types.t;
	parsed: bool; 
}

type program = variable_decl list * function_decl list

let rec string_of_sast_stmt = function
    Block(stmts) ->
      "{\n" ^ String.concat "" (List.map string_of_sast_stmt stmts) ^ "}\n"
  | Return(e) -> "return "


let string_of_sast_v v = 
  "Variable: " ^ v.v_name ^ " = " ^ string_of_obj_type v.v_type ^ "\n" 

let string_of_sast_f f = 
  "Function: " ^ f.ffname ^ " = " ^ string_of_obj_type f.freturntype ^
  String.concat "\n\t" ("\nFormals: " :: (List.map string_of_sast_v f.fformals)) ^
  String.concat "\n\t" ("\nLocals: " :: (List.map string_of_sast_v f.flocals))


let string_of_sast (vars, funcs) = 
  String.concat "" (List.map string_of_sast_v vars) ^ "\n" ^
  String.concat "\n" (List.map string_of_sast_f funcs)
